

#include "ble_pedo_and_camera.h"
#include <string.h>
#include "nordic_common.h"
#include "ble_srv_common.h"
#include "app_util.h"



static void on_connect(ble_pac_t * p_pac, ble_evt_t * p_ble_evt)
{
    p_pac->conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
}



static void on_disconnect(ble_pac_t * p_pac, ble_evt_t * p_ble_evt)
{
    UNUSED_PARAMETER(p_ble_evt);
    p_pac->conn_handle = BLE_CONN_HANDLE_INVALID;
}


static void on_write(ble_pac_t * p_pac, ble_evt_t * p_ble_evt)
{
    ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;
    
    if (
        (p_evt_write->handle == p_pac->UartRx_handles.cccd_handle)
        &&
        (p_evt_write->len == 2)
    )
    {
        uint16_t value = *(uint16_t *) p_evt_write->data;
        if (value == 0x001)
        {
            uint8_t Tbuf[PAC_MAX_DATA_LENGTH];
            
            p_pac->is_uartrx_notification_enabled = true;
            memset(Tbuf, 0, PAC_MAX_DATA_LENGTH);
            ble_pac_uart_send_string(p_pac, Tbuf, PAC_MAX_DATA_LENGTH);
        }
        else
        {
            p_pac->is_uartrx_notification_enabled = false;
        }
        
    }
    
    else if ((p_evt_write->handle == p_pac->UartTx_handles.value_handle) &&
             (p_pac->data_handler != NULL) &&
             (p_pac->DataHandlerContext != NULL))
    {
        p_pac->data_handler(p_pac->DataHandlerContext, p_pac, p_evt_write->data, p_evt_write->len);
    }
#if !defined(_DISABLE_CAMERA_SERVICE)       
    else if (
        (p_evt_write->handle == p_pac->PacRx_handles.cccd_handle)
        &&
        (p_evt_write->len == 2)
    )
    {
        uint16_t value = *(uint16_t *) p_evt_write->data;
        if (value == 0x001)
        {
            uint8_t Tbuf[PAC_MAX_DATA_LENGTH];
            
            p_pac->is_pacrx_notification_enabled = true;
            
            memset(Tbuf, 0, PAC_MAX_DATA_LENGTH);
            ble_pac_send_string(p_pac, Tbuf, PAC_MAX_DATA_LENGTH);
        }
        else
        {
            p_pac->is_pacrx_notification_enabled = false;
        }
        
    }
#endif    
}


void ble_pac_on_ble_evt(ble_pac_t * p_pac, ble_evt_t * p_ble_evt)
{
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            on_connect(p_pac, p_ble_evt);
            break;
            
        case BLE_GAP_EVT_DISCONNECTED:
            on_disconnect(p_pac, p_ble_evt);
            break;
            
        case BLE_GATTS_EVT_WRITE:
            on_write(p_pac, p_ble_evt);
            break;
            
        default:
            break;
    }
}



static uint32_t uart_rx_char_add(ble_pac_t * p_pac, const ble_pac_init_t * p_pac_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    

    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;
    
    memset(&char_md, 0, sizeof(char_md));
    
    char_md.char_props.notify = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
    ble_uuid.type = p_pac->uuid_type;
    ble_uuid.uuid = BLE_UUID_UART_RX_CHARACTERISTIC;
    
    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = 20;
    
    return sd_ble_gatts_characteristic_add(p_pac->service_handle, &char_md,
                                               &attr_char_value,
                                               &p_pac->UartRx_handles);

}


static uint32_t uart_tx_char_add(ble_pac_t * p_pac, const ble_pac_init_t * p_pac_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    
    memset(&char_md, 0, sizeof(char_md));
    
    char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;
    
    ble_uuid.type = p_pac->uuid_type;
    ble_uuid.uuid = BLE_UUID_UART_TX_CHARACTERISTIC;
    
    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 1;
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = 20;
    
    return sd_ble_gatts_characteristic_add(p_pac->service_handle, &char_md,
                                               &attr_char_value,
                                               &p_pac->UartTx_handles);
}


static uint32_t pac_rx_char_add(ble_pac_t * p_pac, const ble_pac_init_t * p_pac_init)
{
#if !defined(_DISABLE_CAMERA_SERVICE)       
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    

    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;
    
    memset(&char_md, 0, sizeof(char_md));
    
    char_md.char_props.notify = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    
    ble_uuid.type = p_pac->uuid_type;
    ble_uuid.uuid = BLE_UUID_PAC_RX_CHARACTERISTIC;
    
    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    
    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = 20;
    
    return sd_ble_gatts_characteristic_add(p_pac->service_handle, &char_md,
                                               &attr_char_value,
                                               &p_pac->PacRx_handles);
#else
    return NRF_SUCCESS;
#endif
}

uint32_t ble_pac_init(ble_pac_t * p_pac, const ble_pac_init_t * p_pac_init)
{
    uint32_t   err_code;
    ble_uuid_t ble_uuid;
    ble_uuid128_t pac_base_uuid = {0x9E, 0xCA, 0xDC, 0x24, 0x0E, 0xE5, 0xA9, 0xE0, 0x93, 0xF3, 0xA3, 0xB5, 0x00, 0x00, 0x40, 0x6E};
    
    // Initialize service structure
    p_pac->conn_handle = BLE_CONN_HANDLE_INVALID;
    p_pac->data_handler = p_pac_init->data_handler;
    p_pac->DataHandlerContext = p_pac_init->p_context;
    p_pac->is_uartrx_notification_enabled = false;
#if !defined(_DISABLE_CAMERA_SERVICE)       
    p_pac->is_pacrx_notification_enabled = false;
#endif
    

    // Add custom base UUID
    err_code = sd_ble_uuid_vs_add(&pac_base_uuid, &p_pac->uuid_type);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }

    // Add service
    ble_uuid.type = p_pac->uuid_type;
    ble_uuid.uuid = BLE_UUID_PAC_SERVICE;

    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &ble_uuid, &p_pac->service_handle);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
    
		//Add characteristic
		err_code = uart_rx_char_add(p_pac, p_pac_init);
		if (err_code != NRF_SUCCESS)
		{
			return err_code;
		}
		
		err_code = uart_tx_char_add(p_pac, p_pac_init);
		if (err_code != NRF_SUCCESS)
		{
			return err_code;
		}
	
        err_code = pac_rx_char_add(p_pac, p_pac_init);
		if (err_code != NRF_SUCCESS)
		{
			return err_code;
		}
		return NRF_SUCCESS;
}

uint32_t ble_pac_uart_send_string(ble_pac_t * p_pac, uint8_t * string, uint16_t length)
{
    ble_gatts_hvx_params_t hvx_params;
    
    if (!p_pac->is_uartrx_notification_enabled)
    {
        return NRF_ERROR_INVALID_STATE;
    }
    
    if (length > PAC_MAX_DATA_LENGTH)
    {
        return NRF_ERROR_INVALID_PARAM;
    }
    
    memset(&hvx_params, 0, sizeof(hvx_params));
    hvx_params.handle = p_pac->UartRx_handles.value_handle;
    hvx_params.p_data = string;
    hvx_params.p_len  = &length;
    hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
    
    return sd_ble_gatts_hvx(p_pac->conn_handle, &hvx_params);
}

uint32_t ble_pac_send_string(ble_pac_t * p_pac, uint8_t * string, uint16_t length)
{
#if !defined(_DISABLE_CAMERA_SERVICE)        
    ble_gatts_hvx_params_t hvx_params;
    
    if (!p_pac->is_pacrx_notification_enabled)
    {
        return NRF_ERROR_INVALID_STATE;
    }
    
    if (length > PAC_MAX_DATA_LENGTH)
    {
        return NRF_ERROR_INVALID_PARAM;
    }
    
    memset(&hvx_params, 0, sizeof(hvx_params));
    hvx_params.handle = p_pac->PacRx_handles.value_handle;
    hvx_params.p_data = string;
    hvx_params.p_len  = &length;
    hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
    
    return sd_ble_gatts_hvx(p_pac->conn_handle, &hvx_params);
#else
    return NRF_SUCCESS;
#endif
}

